CHECKER_NAME = 'biszx-odoo'

FUNCTION_MSG_ID = 90

MSG_DESCRIPTION = (
    'You can review guidelines here: '
    'https://gitlab.com/is4res/biszx-pylint-odoo/READEME.md'
)


TEST_DATA_PATH = 'biszx_pylint_odoo/tests/data'
